# -*- coding: utf-8 -*-

# noinspection SpellCheckingInspection
class NfeTender(object):
    # Tabela de Meios de Pagamento - Publicada em 16/03/2021
    # http://www.nfe.fazenda.gov.br/portal/listaConteudo.aspx?tipoConteudo=Iy/5Qol1YbE=
    # http://www.nfe.fazenda.gov.br/portal/exibirArquivo.aspx?conteudo=SakNjq9TwFA=

    dinheiro = 1
    cheque = 2
    cartao_credito = 3
    cartao_debito = 4
    credito_loja = 5
    vale_alimentacao = 10
    vale_refeicao = 11
    vale_presente = 12
    vale_combustivel = 13
    duplicata_mercanti = 14
    boleto = 15
    deposito_bancario = 16
    pagamento_instantaneo_pix = 17
    transferencia_bancaria__ou__carteira_digital = 18
    programa_fidelidade__ou__cashback__ou__credito_virtual = 19
    sem_pagamento = 90
    outros = 99
