from logging import Logger


class Configurations(object):
    def __init__(self, required_services=None, service_name=None, persistence_name=None, logger=None):
        # type: (str, str, str, Logger) -> None

        self.service_name = service_name
        self.required_services = required_services
        self.persistence_name = persistence_name
        self.logger = logger
