class Table(object):
    def __init__(self, id="", name="", current_pos_id="", status="", tab_id="", business_period="", sector=""):
        # type: (str, str, int, int, bool, int, str) -> None

        self.id = id
        self.name = name
        self.current_pos_id = current_pos_id
        self.status = status
        self.tab_id = tab_id
        self.business_period = business_period
        self.sector = sector
