from abc import ABCMeta, abstractmethod


class NotificationRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def notify(self, name, data):
        # type: (str, str) -> None

        raise NotImplementedError()
